from unittest.mock import Mock

from httpx import patch
import pytest


def mock_subprocess_run_output(output, returncode=0):
    """
    Helper function to create a mock subprocess.run return value with specified output and returncode.
    """
    mock = Mock()
    mock.stdout = output
    mock.returncode = returncode
    return mock



import logging
import warnings
import os

# Disable all warnings
warnings.filterwarnings('ignore')

# Disable specific transformers warnings
os.environ['TRANSFORMERS_NO_ADVISORY_WARNINGS'] = 'true'

# Set logging level to ERROR
logging.getLogger("transformers").setLevel(logging.ERROR)

from typing import List, Dict, Optional
from transformers import AutoTokenizer
import os

from gai.src.utils import get_api_huggingface_key


# Token counter only supports Huggingface models for now
# TODO: Add support for GROQ models
class TokenCounter:
    def __init__(self,
                 model: str
                 ):
        """
        Initialize token counter with specified model using transformers tokenizer.

        Parameters:
        - model: The name or path of the model.
        """
        self.tokens_per_message = 3  # Every message follows {role/name, content}

        # Attempt to load token from environment variable
        hf_token = get_api_huggingface_key()
        if hf_token:
            self.tokenizer = AutoTokenizer.from_pretrained(
                model,
                token=hf_token
            )
        else:
            # Attempt to load without authentication
            self.tokenizer = AutoTokenizer.from_pretrained(model)

    def count_message_tokens(self, message: Dict[str, str]) -> int:
        """
        Count tokens in a single message.
        """
        num_tokens = self.tokens_per_message
        for _, value in message.items():
            value_str = str(value)
            tokens = self.tokenizer.encode(value_str, add_special_tokens=False)
            num_tokens += len(tokens)
        return num_tokens

    def count_tokens(self, messages: List[Dict[str, str]]) -> int:
        """
        Count total tokens in a list of messages.
        """
        try:
            num_tokens = 0
            for message in messages:
                num_tokens += self.count_message_tokens(message)
            # Add 3 tokens for the assistant's reply format (as per OpenAI API)
            num_tokens += 3
            return num_tokens
        except Exception as e:
            raise ValueError(f"Error counting tokens: {str(e)}")

    def adjust_max_tokens(self, user_message: List[Dict[str, str]], max_tokens: int) -> int:
        """
        Calculate remaining tokens based on max_tokens and message tokens.
        """
        try:
            message_tokens = self.count_tokens(user_message)
            remaining_tokens = max_tokens - message_tokens
            if remaining_tokens < 0:
                raise ValueError(f"Message tokens ({message_tokens}) exceed max tokens ({max_tokens})")
            return remaining_tokens
        except Exception as e:
            raise ValueError(f"Error adjusting max tokens: {str(e)}")



import os
import requests
import yaml
import subprocess

from gai.src import Merge_requests, ConfigManager, get_app_name


class Github_api():
    def __init__(self):
        self.Merge_requests = Merge_requests().get_instance()
        self.load_config()

    def load_config(self):
        config_manager = ConfigManager(get_app_name())

        self.repo_owner = self.Merge_requests.get_repo_owner_from_remote_url()
        self.repo_name = self.Merge_requests.get_repo_from_remote_url()
        self.target_branch = config_manager.get_config('target_branch')

    def get_api_key(self):
        api_key = os.environ.get("GITHUB_TOKEN")

        if api_key is None:
            raise ValueError(
                "GITHUB_TOKEN is not set. Please set it in your environment variables.")

        return api_key

    def get_current_branch(self) -> str:
        result = subprocess.run(
            ["git", "rev-parse", "--abbrev-ref", "HEAD"],
            capture_output=True, text=True, check=True
        )
        return result.stdout.strip()

    def create_pull_request(self, title: str, body: str) -> None:
        api_url = f"https://api.github.com/repos/{self.repo_owner}/{self.repo_name}/pulls"

        source_branch = self.get_current_branch()
        api_key = self.get_api_key()

        existing_pr = self.get_existing_pr(api_url=api_url)

        if existing_pr:
            pr_number = existing_pr['number']
            existing_pr_url = f"{api_url}/{pr_number}"
            print(f"A pull request already exists: {existing_pr['html_url']}")
            self.update_pull_request(
                api_url=existing_pr_url,
                title=title,
                body=body
            )
        else:
            data = {
                "title": title,
                "head": source_branch,
                "base": self.target_branch,
                "body": body
            }

            response = requests.post(
                api_url,
                headers={
                    "Authorization": f"token {api_key}",
                    "Accept": "application/vnd.github.v3+json"
                },
                json=data
            )

            if response.status_code == 201:
                print("Pull request created successfully.")
                pr_info = response.json()
                print(f"Pull request URL: {pr_info['html_url']}")
            else:
                print(f"Failed to create pull request: {response.status_code}")
                error_message = response.json()
                print(f"Error message: {error_message}")

    def get_existing_pr(self, api_url: str) -> dict:
        """
        Get existing pull request for the current branch.
        """
        api_key = self.get_api_key()
        source_branch = self.get_current_branch()

        response = requests.get(
            api_url,
            headers={
                "Authorization": f"token {api_key}",
                "Accept": "application/vnd.github.v3+json"
            },
            params={
                "head": f"{self.repo_owner}:{source_branch}",
                "state": "open"
            }
        )

        if response.status_code == 200:
            prs = response.json()
            return prs[0] if prs else None
        return None

    def update_pull_request(self, api_url: str, body: str, title: str) -> None:
        """
        Update an existing pull request.
        """
        api_key = self.get_api_key()

        data = {
            "title": title,
            "body": body
        }

        response = requests.patch(
            api_url,
            headers={
                "Authorization": f"token {api_key}",
                "Accept": "application/vnd.github.v3+json"
            },
            json=data
        )

        if response.status_code == 200:
            print("Pull request updated successfully.")
        else:
            print(f"Failed to update pull request: {response.status_code}")
            error_message = response.json()
            print(f"Error message: {error_message}")
